#include <LiquidCrystal.h>
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

//#define serialPort
#define BTN_UP   1
#define BTN_DOWN 2
#define BTN_LEFT 3
#define BTN_RIGHT 4
#define BTN_SELECT 5
#define BTN_NONE 10

// стоковые параметры для таймера
#define Motor_ESC_Max_THR 2200      // максимальные обороты двигателя
#define Motor_ESC_Normall_THR 2000  // номинальные (полетные) обороты
#define Motor_ESC_Landing_THR 1400  // посадочный режим
#define Motor_ESC_Min_THR 800       // двигатель стоп
#define Time_Of_Calibrate 2         // время между Макс газ и мин газ при необходимости калибровке регулятора
#define Takeoff 120                 // время отведенное на взлет и уборку шасси, в течении этого времени процедура старта может быть отменена (т.е. время до уборки шасси) -  10 секунд 
#define GearUp 40                   // время после старта, начало уборки шасси -  5 секунд 
#define InFly 600                   // время полета в секундах с момента завершения процедуры старта - сейчас стоит 60 секунд
#define Landing 60                  // время отводимое на посадку, в течении этого времени будут выпущены шасси и снижены обороты мотора до посадочных -  60 секунд 
#define delay_g 100                 // коэффициент торможения ))
#define free 0                 


uint8_t stateButtonKey = 0;
int countMenu = 15;
boolean flagButtonState = true;                              // изменилось состояние кнопок
uint8_t lastStateButton = 10;                                 // нет нажатия кнопок
uint64_t timeOfChange;
uint8_t position = 0;
char inputBuffer;
boolean flagTimerReady = false;
boolean flagTimerSend = false;
boolean flagResive = false;
boolean flagSend = false;

uint32_t timeOut = 0;
uint32_t timeSend = 0;
uint8_t i = 0;


int8_t data [14] [4];   // массив для отображения побайтно

char* menu [16] {
  "THR Min Level   ",     // 1 минимальные обороты мотора
  "THR Norm Level  ",     // 2 нормальные обороты мотора
  "THR Max  Level  ",     // 3 максимальные обороты мотора
  "THR Land Level  ",     // 4 обороты мотора при посадке
  "Time Calibrate  ",     // 5 время на калибровку
  "TakeOff time    ",     // 6 врмемя убежать к ручке
  "Time for GearUp ",     // 7 момент уборки шасси с момента старта
  "Flight time     ",     // 8 время полета
  "Landing iime    ",     // 9 время на посадку
  "FREE-1 uint16_t ",     // 10 резервный пункт меню временной параметр
  "FREE-2 uint16_t ",     // 11 резервный пункт меню временной параметр
  "FREE-3 uint16_t ",     // 12 резервный пункт меню временной параметр
  "FREE-1 bollean  ",     // 13 резервный пункт меню логический параметр
  "FREE-2 bollean  ",     // 14 резервный пункт меню логический параметр
  "Write data      ",     // записать данные в таймер
  "Read data       "      // прочитать данные с таймера
};

//************************************************************************************
//  массив настроек программы (скоро уберу)
//************************************************************************************
  uint16_t stack[14] 
  {
  Motor_ESC_Min_THR,          // 1 минимальные обороты мотора
  Motor_ESC_Normall_THR,      // 2 нормальные обороты мотора
  Motor_ESC_Max_THR,          // 3 максимальные обороты мотора
  Motor_ESC_Landing_THR,      // 4 нормальные обороты мотора
  Time_Of_Calibrate,          // 5 обороты мотора при посадке
  Takeoff,                    // 6 время на "убежать к ручке"
  GearUp,                     // 7 момент уборки шасси с момента старта
  InFly,                      // 8 общее время полета с момента старта двигателя
  Landing,                    // 9 время на посадку, сколько времени посадочные обороты
  0,                          // 10 резерв 
  0,                          // 11 резерв 
  0,                          // 12 резерв 
  0,                          // 13 резерв 
  0                           // 14 резерв 
  };


uint8_t *adrStack = (uint8_t*)stack;

int readButton() 
{
  int keyAnalog =  analogRead(A0);
    if (keyAnalog < 50) return BTN_RIGHT; 
    else if (keyAnalog < 150) return BTN_UP;
    else if (keyAnalog < 300) return BTN_DOWN;
    else if (keyAnalog < 550) return BTN_LEFT;
    else if (keyAnalog < 750) return BTN_SELECT;
    else return BTN_NONE;
}

void clearLine(uint8_t line)
{
  lcd.setCursor(0, line);
  lcd.print("                ");
}

void printDisplay(uint8_t pos, uint8_t line, String message)
{
  lcd.setCursor(pos, line);
  lcd.print(message);
}
//******************************************************
// процедура установления связи с таймером
//******************************************************
void setupConnection()
{
  timeSend = millis();
  printDisplay(0, 1, " not connection ");
  while (1)
  {
    if (millis() - timeSend > 200)
    {
      timeSend = millis();
      Serial.write(0x55);          //  передаем запрос на соединение     
    }
    if (Serial.available() > 0)
    {
      inputBuffer = Serial.read();
      if (inputBuffer == 0x55) 
      {
        printDisplay(0, 1, "    Connection  ");
        lcd.setCursor(0, 1);
        return 0;
      }
    }
  }
}
//*******************************************************************************
void readTimerData()
{
  uint8_t step = 0;
  timeOut = millis();

  while (1)
  {
    if (millis() - timeSend >200)
    {
      timeSend = millis();
      Serial.write(0x5A);          //  передаем запрос на соединение     
    }      
    if (Serial.available()>0) 
    {
      inputBuffer = Serial.read (); //  читаем ответ если есть
      adrStack[i] = inputBuffer;
      Serial.print(adrStack[i]);
      i++;
      if (i > 27) 
      {
        return;
      }
    }
  }
}  
//*******************************************************************************
void readData()
{
  uint8_t pos = 0;
  i = 0;
  clearLine(1);
  readTimerData(); 
  for (i = 0; i<13; i++)
  {
    if (i >= 0 && i <= 3) pos = 0;
    else if (i >= 4 && i <= 11) pos = 1;
    else pos = 2;
    switch (pos)
    {
      case 0:
      if (stack[i]>9999) stack[i] = 9999;
      data[i][0] = stack[i]/1000;
      data[i][1] = (stack[i]-data[i][0]*1000)/100;
      data[i][2] = (stack[i]-data[i][0]*1000-data[i][1]*100)/10;
      data[i][3] = (stack[i]-data[i][0]*1000-data[i][1]*100-data[i][2]*10);
      break;
      case 1:
      if (stack[i] > 1800) stack[i] = 1800;
      if (stack[i] < 0) stack[i] = 0;

      data[i][0] = stack[i]/60;                     // минуты
      if (data[i][0] > 60) data[i][0] = 60;
      data[i][3] = (stack[i]-data[i][0]*60)/10;     // десятки секунд
      break;
    }
  }
  printDisplay(0, 1, "    Coomplete   "); 
}
//*****************************************************************
// процедура записи в таймер
//*****************************************************************
void writeData()
{
  printDisplay(0, 1, "write timer data");
  Serial.write(0xA5);
  Serial.write(adrStack,28);
  i=0;
  boolean flagError = false;
  uint8_t tempBuf[28];
  while (i<28)
  {
    if (Serial.available()>0) 
    {
    tempBuf[i] = Serial.read();
    if (tempBuf[i] != adrStack[i]) flagError = true;
    i++;
    }
  }
  if (flagError)   printDisplay(0, 1, "     ERROR      ");
  else   printDisplay(0, 1, "    Coomplete   "); 

}
//*****************************************************************
void setup() 
{
  Serial.begin(9600);
  lcd.begin(16, 2);
  lcd.print("Programmer ready");
  lcd.cursor();
  setupConnection();
  printDisplay(0,0,menu[countMenu]);
}

void loop() 
{
  
  stateButtonKey = readButton();
  if ((stateButtonKey != lastStateButton) && flagButtonState) 
  {
    flagButtonState = false;      // изменилось состояние и флаг поднят
    timeOfChange = millis();
  }
    if (millis() - timeOfChange > 120  && (stateButtonKey != lastStateButton)) 
    {
      lastStateButton = stateButtonKey;
      flagButtonState = true;
      switch (stateButtonKey) 
      {
//***********************************************************************
        case BTN_UP:
        if (countMenu >= 0 && countMenu <= 3) 
        {
          data [countMenu][position]++;
          if (data [countMenu][position] > 9) data [countMenu][position] = 0;
          lcd.setCursor(position, 1);
          lcd.print(data [countMenu][position]);
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 4 && countMenu <= 11) 
        {
          data [countMenu][position]++;
          if (position == 0 && data [countMenu][position] > 30) data [countMenu][position] = 0;
          if (position == 3 && data [countMenu][position] > 5) data [countMenu][position] = 0;
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.setCursor(2, 1);
          lcd.print(":");
          lcd.print(data[countMenu][3]);
          lcd.print("0 m:s");
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 12 && countMenu <= 13) 
        {
          if (data [countMenu][0] == 0) data [countMenu][0] = 1; else data [countMenu][0] = 0;
          lcd.setCursor(0, 1);
          if (data [countMenu][0] == 0) lcd.print("False       "); else lcd.print("True        "); 
          lcd.setCursor(2, 1);
        }
        break;
//*********************************************************************
        case BTN_DOWN:
        if (countMenu >= 0 && countMenu <= 3) 
        {
          data [countMenu][position]--;
          if (data [countMenu][position] < 0) data [countMenu][position] = 9;
          lcd.setCursor(position, 1);
          lcd.print(data [countMenu][position]);
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 4 && countMenu <= 11) 
        {
          data [countMenu][position]--;
          if (position == 0 && data [countMenu][position] < 0) data [countMenu][position] = 30;
          if (position == 3 && data [countMenu][position] < 0) data [countMenu][position] = 5;
//          printDisplay(0,1,"               ");
//          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.setCursor(2, 1);
          lcd.print(":");
          lcd.print(data[countMenu][3]);
          lcd.print("0 m:s");
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 12 && countMenu <= 13) 
        {
          if (data [countMenu][0] == 0) data [countMenu][0] = 1; else data [countMenu][0] = 0;
          lcd.setCursor(0, 1);
          if (data [countMenu][0] == 0) lcd.print("False       "); else lcd.print("True        "); 
          lcd.setCursor(2, 1);
        }
        break;
//*********************************************************************
        case BTN_LEFT:
        countMenu--;
        clearLine(1);
        if (countMenu < 0) countMenu = 15;
        printDisplay(0,0,menu[countMenu]);
        if (countMenu >= 0 && countMenu <= 3)
        {
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.print(data[countMenu][1]);
          lcd.print(data[countMenu][2]);
          lcd.print(data[countMenu][3]);
          lcd.setCursor(0, 1);
        }
        if (countMenu >= 4 && countMenu <= 11)
        {
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.setCursor(2, 1);
          lcd.print(":");
          lcd.print(data[countMenu][3]);
          lcd.print("0 m:s");
          lcd.setCursor(2, 1);
        }
        if (countMenu >= 12 && countMenu <= 13)
        {
          if (data [countMenu][0] == 0) 
          {
            printDisplay(0,1,"False        ");
          }
          else
          {
            printDisplay(0,1,"True         ");
          }
        }
        if (countMenu >= 14 && countMenu <= 15)
        {
        }
        position = 0;
        lcd.setCursor(position, 1);
        break;
//**************************************************************        
        case BTN_RIGHT:
        countMenu++;
        clearLine(1);
        if (countMenu > 15) countMenu = 0;
        printDisplay(0,0,menu[countMenu]);
        if (countMenu >= 0 && countMenu <= 3)
        {
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.print(data[countMenu][1]);
          lcd.print(data[countMenu][2]);
          lcd.print(data[countMenu][3]);
          lcd.setCursor(0, 1);
        }
        if (countMenu >= 4 && countMenu <= 11)
        {
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.setCursor(2, 1);
          lcd.print(":");
          lcd.print(data[countMenu][3]);
          lcd.print("0 m:s");
          lcd.setCursor(2, 1);
        }
        if (countMenu >= 12 && countMenu <= 13)
        {
          if (data [countMenu][0] == 0) 
          {
            printDisplay(0,1,"False        ");
          }
          else
          {
            printDisplay(0,1,"True         ");
          }
        }
        if (countMenu >= 14 && countMenu <= 15)
        {
        }
        position = 0;
        lcd.setCursor(position, 1);
        break;
//****************************************************************************
        case BTN_SELECT:
        if (countMenu == 14) {writeData(); break;}
        if (countMenu == 15) {readData(); break;}
        if (countMenu >= 0 && countMenu <=3) 
        {
          position++;
          if (position > 3) position = 0;
          lcd.setCursor(position, 1);

        }
        if (countMenu >= 4 && countMenu <= 11) 
        {
          if (position == 0) position = 3; else position = 0;
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 12 && countMenu <= 13) 
        {
          position = 0;
          lcd.setCursor(position, 1);
        }
        break;
      }
    }
}
